@@ -243,7 +243,8 @@ def init_inference(model,
                    args=None,
                    enable_cuda_graph=False,
                    save_mp_checkpoint_path=None,
-                   base_dir=""):
+                   base_dir="",
+                   max_tokens=1024):
     """Initialize the DeepSpeed InferenceEngine.
 
     Arguments:
@@ -292,7 +293,8 @@ def init_inference(model,
         save_mp_checkpoint_path: The path for which we want to save the loaded model with a checkpoint. This feature is used for adjusting the
             parallelism degree to help alleviate the model loading overhead. It does not save any new checkpoint if no path is passed.
         base_dir: This shows the root directory under which all the checkpoint files exists. This can be passed through the json config too.
-
+        max_tokens: This argument shows the maximum number of tokens inference-engine can work with, including the input and output tokens.
+            Please consider increasing it to the required token-length required for your use-case.
     Returns:
         A deepspeed.InferenceEngine wrapped model.
     """
@@ -323,6 +325,7 @@ def init_inference(model,
                              args,
                              enable_cuda_graph,
                              save_mp_checkpoint_path,
-                             base_dir)
+                             base_dir,
+                             max_tokens)
 
     return engine
