@@ -11,6 +11,7 @@ from deepspeed.utils.logging import log_dist
 from torch.nn.modules import Module
 from packaging import version as pkg_version
 from deepspeed.runtime.checkpoint_engine.torch_checkpoint_engine import TorchCheckpointEngine
+from deepspeed.utils.timer import SynchronizedWallClockTimer
 
 from ..runtime.state_dict_factory import SDLoaderFactory
 from ..runtime.weight_quantizer import WeightQuantization
@@ -25,6 +26,8 @@ from ..module_inject.replace_policy import DSPolicy
 DS_INFERENCE_ENABLED = False
 from torch import nn
 
+INFERENCE_MODEL_TIMER = "model-forward-inference"
+
 
 class InferenceEngine(Module):
     inference_mp_group = None
@@ -53,7 +56,8 @@ class InferenceEngine(Module):
                  config=None,
                  enable_cuda_graph=False,
                  save_mp_checkpoint_path=None,
-                 base_dir=""):
+                 base_dir="",
+                 max_out_tokens=1024):
         """
         Args:
             model: torch.nn.Module
@@ -144,7 +148,8 @@ class InferenceEngine(Module):
                     training_mp_size,
                     self.checkpoint if replace_with_kernel_inject else None,
                     save_mp_checkpoint_path=save_mp_checkpoint_path,
-                    base_dir=base_dir)
+                    base_dir=base_dir,
+                    max_out_tokens=max_out_tokens)
         elif replace_method == 'auto':
             self._apply_injection_policy(
                 return_tuple=return_tuple,
@@ -155,7 +160,8 @@ class InferenceEngine(Module):
                 training_mp_size=training_mp_size,
                 checkpoint_dir=self.checkpoint if replace_with_kernel_inject else None,
                 save_mp_checkpoint_path=save_mp_checkpoint_path,
-                base_dir=base_dir)
+                base_dir=base_dir,
+                max_out_tokens=max_out_tokens)
 
         device = torch.cuda.current_device()
         self.module.to(device)
@@ -168,11 +174,14 @@ class InferenceEngine(Module):
         if self.mp_world_size > 1:
             assert not self.enable_cuda_graph, "Cuda graph is not supported for model parallelism"
 
-    def profile_model_time(self):
+    def profile_model_time(self, use_cuda_events=True):
         if not self.model_profile_enabled and not self.enable_cuda_graph:
             self.module.register_forward_pre_hook(self._pre_forward_hook)
             self.module.register_forward_hook(self._post_forward_hook)
         self.model_profile_enabled = True
+        self.use_cuda_events = use_cuda_events
+        if self.use_cuda_events:
+            self.timers = SynchronizedWallClockTimer()
 
     def _get_model_config_generate(self, config):
         self.config = getattr(self.module, 'config', None) if config is None else config
@@ -184,13 +193,21 @@ class InferenceEngine(Module):
                 self.module.transformer._prepare_attn_mask = lambda attention_mask, *args, **kwargs: attention_mask
 
     def _pre_forward_hook(self, module, *inputs, **kwargs):
-        torch.cuda.synchronize()
-        self._start = time.time()
+        if self.use_cuda_events:
+            self.timers(INFERENCE_MODEL_TIMER).start()
+        else:
+            torch.cuda.synchronize()
+            self._start = time.time()
 
     def _post_forward_hook(self, module, input, output):
-        torch.cuda.synchronize()
-        self._end = time.time()
-        self._model_times.append(self._end - self._start)
+        if self.use_cuda_events:
+            self.timers(INFERENCE_MODEL_TIMER).stop()
+            elapsed_time = self.timers(INFERENCE_MODEL_TIMER).elapsed(reset=True)
+        else:
+            torch.cuda.synchronize()
+            self._end = time.time()
+            elapsed_time = self._end - self._start
+        self._model_times.append(elapsed_time)
 
     def _create_model_parallel_group(self):
         # Call the init process
@@ -355,13 +372,15 @@ class InferenceEngine(Module):
                                 training_mp_size=1,
                                 checkpoint_dir=None,
                                 save_mp_checkpoint_path=False,
-                                base_dir=""):
+                                base_dir="",
+                                max_out_tokens=1024):
         checkpoint = SDLoaderFactory.get_sd_loader_json(
             checkpoint_dir,
             self.checkpoint_engine) if checkpoint_dir is not None else None
 
         generic_injection(self.module,
-                          fp16=(self.dtype == torch.half) or (self.dtype == torch.int8))
+                          fp16=(self.dtype == torch.half) or (self.dtype == torch.int8),
+                          enable_cuda_graph=self.enable_cuda_graph)
 
         if isinstance(self.module, torch.nn.Module):
             replace_transformer_layer(
@@ -390,7 +409,8 @@ class InferenceEngine(Module):
                 checkpoint_dict=checkpoint,
                 save_mp_checkpoint_path=save_mp_checkpoint_path,
                 base_dir=base_dir,
-                enable_cuda_graph=self.enable_cuda_graph)
+                enable_cuda_graph=self.enable_cuda_graph,
+                max_out_tokens=max_out_tokens)
 
     def _get_all_ckpt_names(self, checkpoints_path, tag):
         ckpt_file_pattern = self._get_ckpt_name(checkpoints_path,
@@ -430,7 +450,8 @@ class InferenceEngine(Module):
             ckpt_list = self._get_all_ckpt_names(load_dir, tag)
             sd_loader = SDLoaderFactory.get_sd_loader(ckpt_list, self.checkpoint_engine)
         else:
-            sd_loader = SDLoaderFactory.get_sd_loader_json(load_dir)
+            sd_loader = SDLoaderFactory.get_sd_loader_json(load_dir,
+                                                           self.checkpoint_engine)
 
         if type(sd_loader) is list:
             self.sd = torch.load(sd_loader[0], map_location='cpu')
@@ -473,7 +494,6 @@ class InferenceEngine(Module):
 
             self.module.load_state_dict(
                 state_dict=checkpoint[self._choose_module_key(checkpoint)],
-                checkpoint_engine=self.checkpoint_engine,
                 strict=load_module_strict)
 
     def _choose_module_key(self, sd):
