@@ -195,8 +195,8 @@ def _module_match(module):
     return None
 
 
-def generic_injection(module, fp16=False):
-    def replace_attn(child, policy, layer_id):
+def generic_injection(module, fp16=False, enable_cuda_graph=True):
+    def replace_attn(child, policy):
         policy_attn = policy.attention(child)
         if policy_attn is None:
             return child
@@ -210,10 +210,12 @@ def generic_injection(module, fp16=False):
             heads=heads,
             fp16=fp16,
             triangular_masking=False,
+            max_out_tokens=4096,
         )
-        attn_module = transformer_inference.DeepSpeedAttention(config)
+        attn_module = transformer_inference.DeepSpeedDiffusersAttention(config)
 
         def transpose(data):
+            data = data.contiguous()
             data.reshape(-1).copy_(data.transpose(-1, -2).contiguous().reshape(-1))
             data = data.reshape(data.shape[-1], data.shape[-2])
             data.to(torch.cuda.current_device())
@@ -232,13 +234,24 @@ def generic_injection(module, fp16=False):
         attn_module.attn_ob.data.copy_(attn_ob.data.to(torch.cuda.current_device()))
         return attn_module
 
+    def replace_attn_block(child, policy):
+        config = transformer_inference.Diffusers2DTransformerConfig()
+        return transformer_inference.DeepSpeedDiffusersTransformerBlock(child, config)
+
     if isinstance(module, torch.nn.Module):
         pass
     else:
+        if fp16 is False:
+            raise ValueError("Generic injection only supported with FP16")
+
         try:
             import diffusers
             cross_attention = diffusers.models.attention.CrossAttention
-            new_policies = {cross_attention: replace_attn}
+            attention_block = diffusers.models.attention.BasicTransformerBlock
+            new_policies = {
+                cross_attention: replace_attn,
+                attention_block: replace_attn_block,
+            }
         except ImportError:
             new_policies = {}
 
@@ -246,30 +259,29 @@ def generic_injection(module, fp16=False):
         #                          module.text_encoder,
         #                          training=False,
         #                          replace_with_kernel_inject=True,
-        #                          triangular_masking=True)
-        #from .encoder import DSClipEncoder
-        #cg_encoder = DSClipEncoder(module.text_encoder)
-        #setattr(module, 'text_encoder', cg_encoder)
+        #                          triangular_masking=True,
+        #                          max_out_tokens=8192)
+        from ..model_implementations.transformers.clip_encoder import DSClipEncoder
+        cg_encoder = DSClipEncoder(module.text_encoder,
+                                   enable_cuda_graph=enable_cuda_graph)
+        setattr(module, 'text_encoder', cg_encoder)
         for name in module.__dict__.keys():
             sub_module = getattr(module, name)
             policy = _module_match(sub_module)
 
             if policy is not None:
 
-                def _replace_module(module, policy, layer_id=0):
+                def _replace_module(module, policy):
                     for name, child in module.named_children():
+                        _replace_module(child, policy)
                         if child.__class__ in new_policies:
                             replaced_module = new_policies[child.__class__](child,
-                                                                            policy,
-                                                                            layer_id)
+                                                                            policy)
                             setattr(module, name, replaced_module)
-                            layer_id += 1
-                        else:
-                            layer_id = _replace_module(child, policy, layer_id=layer_id)
-                    return layer_id
 
                 _replace_module(sub_module, policy)
-                new_module = policy.apply(sub_module)
+                new_module = policy.apply(sub_module,
+                                          enable_cuda_graph=enable_cuda_graph)
                 print(f"**** found and replaced {name} w. {type(new_module)}")
                 setattr(module, name, new_module)
 
@@ -303,7 +315,8 @@ def replace_transformer_layer(orig_layer_impl,
                               checkpoint_dict=None,
                               save_mp_checkpoint_path=None,
                               base_dir="",
-                              enable_cuda_graph=False):
+                              enable_cuda_graph=False,
+                              max_out_tokens=1024):
     """ Replace bert-style transformer layers with DeepSpeed's transformer layer
     Arguments:
         orig_layer_impl (torch.nn.Module): the original transformer layer implementation to look for,
@@ -404,6 +417,9 @@ def replace_transformer_layer(orig_layer_impl,
 
         quantizer = GroupQuantizer(q_int8=quantize)
         if inference:
+            scale_attn_by_inverse_layer_idx = config.scale_attn_by_inverse_layer_idx if hasattr(
+                config,
+                'scale_attn_by_inverse_layer_idx') else False
             if moe:
                 ep_world_size = dist.get_world_size()
                 local_ep_size = 1 if num_experts < ep_world_size else num_experts // ep_world_size
@@ -420,7 +436,8 @@ def replace_transformer_layer(orig_layer_impl,
                     q_int8=quantize,
                     moe_experts=local_ep_size,
                     global_experts=num_experts,
-                    mlp_type=moe_type)
+                    mlp_type=moe_type,
+                    scale_attn_by_inverse_layer_idx=scale_attn_by_inverse_layer_idx)
             else:
                 rotary_dim = config.rotary_dim if hasattr(config, 'rotary_dim') else child.attention.rotary_ndims \
                                             if hasattr(child, 'attention') and hasattr(child.attention,'rotary_ndims') else -1
@@ -451,7 +468,9 @@ def replace_transformer_layer(orig_layer_impl,
                     mlp_after_attn=(rotary_dim is None or rotary_dim < 0),
                     mlp_act_func_type=policy.mlp_act_func_type,
                     training_mp_size=training_mp_size,
-                    bigscience_bloom=bigscience_bloom)
+                    bigscience_bloom=bigscience_bloom,
+                    max_out_tokens=max_out_tokens,
+                    scale_attn_by_inverse_layer_idx=scale_attn_by_inverse_layer_idx)
 
             if quantize and quantize_settings is not None:
                 (quantization_scales,
@@ -513,7 +532,7 @@ def replace_transformer_layer(orig_layer_impl,
             # transpose it here to reduce inference cost!
             def transpose(data):
                 # temp move to cpu to avoid requiring extra GPU memory during the reshape
-                data = data.to('cpu')
+                data = data.to('cpu').contiguous()
                 data.reshape(-1).copy_(data.transpose(-1, -2).contiguous().reshape(-1))
                 data = data.reshape(data.shape[-1], data.shape[-2])
                 data.to(torch.cuda.current_device())
